{
	T7F - A program to read and set T7F channels
	Copyright (C) 2000 Giuseppe Perotti, I1EPJ

	The T7F is a 70 cm FM/FSK Transceiver by Holger Eckardt, DF2FQ

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
}

unit t7fu1;

interface

uses
  SysUtils, QForms, t7fabout, gnulic, t7fpinout, QDialogs, QMenus, QTypes,
  QGraphics, QStdCtrls, QMask, QControls, Classes, LibC;

const OneSec = 1/86400;

type
  TForm1 = class(TForm)
	Label1: TLabel;
	Label2: TLabel;
	Label3: TLabel;
	Label4: TLabel;
	Label5: TLabel;
	Label6: TLabel;
	Label7: TLabel;
	Label8: TLabel;
	Label9: TLabel;
	Label10: TLabel;
	MainMenu1: TMainMenu;
	Label11: TLabel;
	Label12: TLabel;
	File1: TMenuItem;
        Load1: TMenuItem;
	Save1: TMenuItem;
	Exit1: TMenuItem;
	T7F1: TMenuItem;
	Readsetup1: TMenuItem;
        WriteChanToT7F1: TMenuItem;
        Help1: TMenuItem;
        About1: TMenuItem;
	ComPort1: TMenuItem;
	COM11: TMenuItem;
	COM21: TMenuItem;
	COM31: TMenuItem;
	COM41: TMenuItem;
        OpenDialog1: TOpenDialog;
	SaveDialog1: TSaveDialog;
	HexDump1: TMenuItem;
	Rx0: TMaskEdit;
        Tx0: TMaskEdit;
        Rx1: TMaskEdit;
	Tx1: TMaskEdit;
        Rx2: TMaskEdit;
	Tx2: TMaskEdit;
	Rx3: TMaskEdit;
	Tx3: TMaskEdit;
	Rx4: TMaskEdit;
        Tx4: TMaskEdit;
        Rx5: TMaskEdit;
        Tx5: TMaskEdit;
	Rx6: TMaskEdit;
        Tx6: TMaskEdit;
	Rx7: TMaskEdit;
	Tx7: TMaskEdit;
	Rx8: TMaskEdit;
	Tx8: TMaskEdit;
	Rx9: TMaskEdit;
        Tx9: TMaskEdit;
        ChannelSpacing1: TMenuItem;
        N125kHz1: TMenuItem;
	N25kHz1: TMenuItem;
        GNULicense1: TMenuItem;
        Connectorspinout1: TMenuItem;
    New1: TMenuItem;
        procedure SetComParams;
	procedure COM11Click(Sender: TObject);
	procedure COM21Click(Sender: TObject);
	procedure COM31Click(Sender: TObject);
        procedure COM41Click(Sender: TObject);
	procedure Readsetup1Click(Sender: TObject);
        procedure WriteChanToT7F1Click(Sender: TObject);
	procedure Exit1Click(Sender: TObject);
        procedure FormCreate(Sender: TObject);
	procedure RxTxEnter(Sender: TObject);
	procedure N125kHz1Click(Sender: TObject);
	procedure N25kHz1Click(Sender: TObject);
	procedure Load1Click(Sender: TObject);
	procedure Save1Click(Sender: TObject);
	procedure CheckFreq(EdBox: TMaskEdit);
	procedure HexDump1Click(Sender: TObject);
        procedure RxTxExit(Sender: TObject);
        procedure GNULicense1Click(Sender: TObject);
        procedure Connectorspinout1Click(Sender: TObject);
        procedure FormResize(Sender: TObject);
        procedure New1Click(Sender: TObject);
        procedure About1Click(Sender: TObject);

  private
	 { Private declarations }
	 function ReadChannels: Boolean;
         procedure EmptyBuffer;
	 procedure ParseChannels;
	 procedure DisplayChannels;
         procedure ClearDisplay;
	 procedure GetFreq;
	 procedure SaveConfig;

  public
	 { Public declarations }
  end;

  TRxTxCh = array[0..9] of integer;

  TChanConf = record
        ChanSpc: integer;
        RxCh: TRxTxCh;
        TxCh: TRxTxCh;
  end;

var
  Form1: TForm1;
  RxBuf: array[0..255] of char;
  sz: array[0..40] of char;
  RxCh, TXCh: TRxTxCh;
  ChanSpc: integer;
  ChanConf: TChanConf;
  DataReceived: boolean;
  DataValid: Boolean;
  TTYhandle: integer;
  TTYPort: integer;
  tio: termios;

implementation

uses hexd;

{$R *.xfm}

{
 Init RS232 port parameters
}
procedure TForm1.SetComParams;

begin
        tcgetattr(TTYHandle,tio);
        cfmakeraw(tio);
        cfsetospeed(tio, B1200);
        cfsetispeed(tio, B1200);
        tcsetattr(TTYHandle,TCSANOW,tio);
end;

{
	Select COM1
}
procedure TForm1.COM11Click(Sender: TObject);
begin
        if TTYHandle > 0 then FileClose(TTYHandle);
        TTYPort := 0;
        TTYHandle := FileOpen('/dev/ttyS0', O_RDWR);
        if TTYHandle >=0 then begin
                SetComParams;
                tcsetattr(TTYHandle,TCSANOW,tio);
	        Com11.Checked := True;
	        Com21.Checked := False;
	        Com31.Checked := False;
	        Com41.Checked := False;
	        SaveConfig;
        end else begin
                StrFmt(sz, 'Can''t open ttyS%d',[TTYPort]);
	        Application.MessageBox(sz,'Error!',[smbOK],smsCritical);
        end;
end;

{
	Select COM2
}
procedure TForm1.COM21Click(Sender: TObject);
begin
        if TTYHandle > 0 then FileClose(TTYHandle);
        TTYPort := 1;
        TTYHandle := FileOpen('/dev/ttyS1', O_RDWR);
        if TTYHandle >=0 then begin
                SetComParams;
	        Com11.Checked := False;
	        Com21.Checked := True;
	        Com31.Checked := False;
	        Com41.Checked := False;
	        SaveConfig;
        end else begin
                StrFmt(sz, 'Can''t open ttyS%d',[TTYPort]);
	        Application.MessageBox(sz,'Error!',[smbOK],smsCritical);
        end;
end;

{
	Select COM3
}
procedure TForm1.COM31Click(Sender: TObject);
begin
        if TTYHandle > 0 then FileClose(TTYHandle);
        TTYPort := 2;
        TTYHandle := FileOpen('/dev/ttyS2', O_RDWR);
        if TTYHandle >=0 then begin
                SetComParams;
	        Com11.Checked := False;
	        Com21.Checked := False;
	        Com31.Checked := True;
	        Com41.Checked := False;
	        SaveConfig;
        end else begin
                StrFmt(sz, 'Can''t open ttyS%d',[TTYPort]);
	        Application.MessageBox(sz,'Error!',[smbOK],smsCritical);
        end;
end;

{
	Select COM4
}
procedure TForm1.COM41Click(Sender: TObject);
begin
        if TTYHandle > 0 then FileClose(TTYHandle);
        TTYPort := 3;
        TTYHandle := FileOpen('/dev/ttyS3', O_RDWR);
        if TTYHandle >= 0 then begin
                SetComParams;
                Com11.Checked := False;
                Com21.Checked := False;
                Com31.Checked := False;
                Com41.Checked := True;
                SaveConfig;
        end else begin
                StrFmt(sz, 'Can''t open ttyS%d',[TTYPort]);
	        Application.MessageBox(sz,'Error!',[smbOK],smsCritical);
        end;
end;

{
	Save configuration data (COM port number and default channel spacing)
}
procedure Tform1.SaveConfig;

var ConfFile: FILE of integer;
	 comport: integer;

begin
	try
            AssignFile(ConfFile, 'T7F.INI');
            Rewrite(ConfFile);
	    comport := TTYPort;
	    Write(ConfFile, comport);
	    Write(ConfFile, ChanSpc);
	    CloseFile(ConfFile);
	except
            on Exception do begin
		Application.MessageBox('Can''t write configuration file',
		'Error!',[smbOK],smsCritical);
            end;
	end;
end;

{
	Read channel assignment for T7F
	Issue an E command then wait 2s to allow response to come in
}
function Tform1.ReadChannels: boolean;

var t: TDateTime;
    DumpCmd: char;
    nv, nc: integer;
    nfd: PPollFD;

begin
        GetMem(nfd, sizeof(PPollFD));
        DumpCmd := 'E';
        nv := 0;
        nc := 0;
	FillChar(RxBuf,256,0);
        repeat
            nv := nv + 1;
            FileWrite(TTYHandle, DumpCmd, 1);
	    t := time;
            repeat
	        Application.ProcessMessages;
	    until (time-t) > 2*OneSec;
            nfd.fd := TTYHandle;
            nfd.events := POLLIN;
            Poll(nfd, 1, 0);
            if nfd.revents = POLLIN then
            	nc := FileRead(TTYHandle, RxBuf, 256);
        until (nc = 112) or (nv = 5);
        if nv = 5 then begin
	    Application.MessageBox(
                'T7F not responding!','Warning!',[smbOK], smsWarning);
            DataValid := False;
            ReadChannels := False;
            ClearDisplay;
        end else begin
            DataValid := True;
            ReadChannels := True;
        end;
        FreeMem(nfd);
end;

{
        Empty receive buffer
        Reads on until no more characters are available
}
procedure Tform1.EmptyBuffer;

var
    nfd: PPollFD;
    Trash: array[0..255] of char;

begin
    GetMem(nfd, sizeof(PPollFD));
    repeat
        nfd.fd := TTYHandle;
        nfd.events := POLLIN;
        poll(nfd, 1, 100);
        if nfd.revents = POLLIN then FileRead(TTYHandle, Trash, 256);
    until nfd.revents = 0;
end;

{
	Parse T7F channel hex dump
	(10 lines of the form ' tttt rrrr' where
		tttt is the transmit divisor
		rrrr is the receive divisor
}
procedure Tform1.ParseChannels;

var n: integer;
    p: pchar;

{ Convert a 4 digit hex number starting at pch to integer }
function Convhex4(pch: pchar):integer;

var n,res,digit: integer;
             p1: pchar;

begin
	p1 := pch;
	res := 0;
	for n := 0 to 3 do begin
		if ord(p1^) <= ord('9') then begin
			digit := ord(p1^) - ord('0');
		end else begin
			digit := ord(p1^) - ord('A') + 10;
		end;
		res := 16*res+digit;
		inc(p1);
	end;
	ConvHex4 := res;
end;

begin
	p := RxBuf;
	for n := 0 to 9 do begin
		while (p^=' ') or (p^=chr(13)) or (p^=chr(10)) or (p^='E') do inc(p);
		TxCh[n] := ConvHex4(p);
		inc(p,4);
		while (p^=' ') or (p^=chr(13)) or (p^=chr(10)) or (p^='E') do inc(p);
		RxCh[n] := ConvHex4(p);
		inc(p,4);
	end;
end;

{
	Display channels assignment
}
procedure Tform1.DisplayChannels;

var MHz, kHz, Hz, fr: longint;
	 s: string;

{ Converts divisor value to frequency.
  All frequencies troughout the program are expressed in Hz }
function DivToFreq(divisor: integer): LongInt;

begin
        if (divisor >=0) and
           ((ChanSpc = 12500) and (divisor <= 791)) or
           ((ChanSpc = 25000) and (divisor <= 395)) then
	        DivToFreq := 430000000 + LongInt(divisor)*LongInt(ChanSpc)
        else
                DivToFreq := -1;
end;

begin
	fr := DivToFreq(RxCh[0]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx0.Text := s;
	fr := DivToFreq(TxCh[0]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx0.Text := s;

	fr := DivToFreq(RxCh[1]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx1.Text := s;
	fr := DivToFreq(TxCh[1]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx1.Text := s;

	fr := DivToFreq(RxCh[2]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx2.Text := s;
	fr := DivToFreq(TxCh[2]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx2.Text := s;

	fr := DivToFreq(RxCh[3]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx3.Text := s;
	fr := DivToFreq(TxCh[3]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx3.Text := s;

	fr := DivToFreq(RxCh[4]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx4.Text := s;
	fr := DivToFreq(TxCh[4]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx4.Text := s;

	fr := DivToFreq(RxCh[5]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx5.Text := s;
	fr := DivToFreq(TxCh[5]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx5.Text := s;

	fr := DivToFreq(RxCh[6]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx6.Text := s;
        fr := DivToFreq(TxCh[6]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx6.Text := s;

	fr := DivToFreq(RxCh[7]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx7.Text := s;
        fr := DivToFreq(TxCh[7]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx7.Text := s;

	fr := DivToFreq(RxCh[8]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx8.Text := s;
        fr := DivToFreq(TxCh[8]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx8.Text := s;

	fr := DivToFreq(RxCh[9]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Rx9.Text := s;
        fr := DivToFreq(TxCh[9]);
	MHz := fr div 1000000;
	kHz := (fr mod 1000000) div 1000;
	Hz := (fr mod 1000) div 100;
	FmtStr(s, '%3d.%3.3d.%1d',[MHz,kHz,Hz]);
	Tx9.Text := s;
end;

procedure Tform1.ClearDisplay;
begin
        Rx0.Clear;
        Tx0.Clear;
        Rx1.Clear;
        Tx1.Clear;
        Rx2.Clear;
        Tx2.Clear;
        Rx3.Clear;
        Tx3.Clear;
        Rx4.Clear;
        Tx4.Clear;
        Rx5.Clear;
        Tx5.Clear;
        Rx6.Clear;
        Tx6.Clear;
        Rx7.Clear;
        Tx7.Clear;
        Rx8.Clear;
        Tx8.Clear;
        Rx9.Clear;
        Tx9.Clear;
end;
{
	Read, parse and display channel data from T7F
}
procedure TForm1.ReadSetup1Click(Sender: TObject);

begin
	if ReadChannels then begin
            ParseChannels;
            DisplayChannels;
        end;
end;

{
	Get new frequency informations from controls
}
procedure Tform1.GetFreq;

{ Convert MHz,kHz and Hz data to frequency.
  All frequencies troughout the program are expressed in Hz
}
function ReadFreq(f: string): longint;

var Errc: integer;
	 MHz, kHz, Hz, fr: longint;

begin
        fr := 0;
        if f <> '' then begin
        	Val(Copy(f, 1, 3), MHz, ErrC);
	        Val(Copy(f, 5, 3), kHz, ErrC);
        	Val(Copy(f, 9, 1), Hz, ErrC);
	        fr := 1000000*MHz+1000*kHz+100*Hz;
        end;
      	ReadFreq := fr;
end;

begin
	RxCh[0] := (ReadFreq(Rx0.Text)-430000000) div ChanSpc;
	TxCh[0] := (ReadFreq(Tx0.text)-430000000) div ChanSpc;
	RxCh[1] := (ReadFreq(Rx1.Text)-430000000) div ChanSpc;
	TxCh[1] := (ReadFreq(Tx1.text)-430000000) div ChanSpc;
	RxCh[2] := (ReadFreq(Rx2.Text)-430000000) div ChanSpc;
	TxCh[2] := (ReadFreq(Tx2.text)-430000000) div ChanSpc;
	RxCh[3] := (ReadFreq(Rx3.Text)-430000000) div ChanSpc;
	TxCh[3] := (ReadFreq(Tx3.text)-430000000) div ChanSpc;
	RxCh[4] := (ReadFreq(Rx4.Text)-430000000) div ChanSpc;
	TxCh[4] := (ReadFreq(Tx4.text)-430000000) div ChanSpc;
	RxCh[5] := (ReadFreq(Rx5.Text)-430000000) div ChanSpc;
	TxCh[5] := (ReadFreq(Tx5.text)-430000000) div ChanSpc;
	RxCh[6] := (ReadFreq(Rx6.Text)-430000000) div ChanSpc;
	TxCh[6] := (ReadFreq(Tx6.text)-430000000) div ChanSpc;
	RxCh[7] := (ReadFreq(Rx7.Text)-430000000) div ChanSpc;
	TxCh[7] := (ReadFreq(Tx7.text)-430000000) div ChanSpc;
	RxCh[8] := (ReadFreq(Rx8.Text)-430000000) div ChanSpc;
	TxCh[8] := (ReadFreq(Tx8.text)-430000000) div ChanSpc;
	RxCh[9] := (ReadFreq(Rx9.Text)-430000000) div ChanSpc;
	TxCh[9] := (ReadFreq(Tx9.text)-430000000) div ChanSpc;
end;

{
	Write channel informations to T7F
}
procedure TForm1.WriteChanToT7F1Click(Sender: TObject);

var n,nc: integer;
    buf: array[0..255] of char;
    ch: char;
    t: TDateTime;

 begin
     if DataValid then begin
	GetFreq;
	for n := 0 to 9 do begin
            if (RxCh[n] >=0) and (TxCh[n] >= 0) then begin
		StrFmt(buf, 'C%d%3.3d%3.3d%s',[n,TxCh[n],RxCh[n],chr(13)]);
		case n of
			0: begin label1.color := clRed; end;
			1: begin label2.color := clRED; label1.color := clBtnFace end;
			2: begin label3.color := clRED; label2.color := clBtnFace end;
			3: begin label4.color := clRED; label3.color := clBtnFace end;
			4: begin label5.color := clRED; label4.color := clBtnFace end;
			5: begin label6.color := clRED; label5.color := clBtnFace end;
			6: begin label7.color := clRED; label6.color := clBtnFace end;
			7: begin label8.color := clRED; label7.color := clBtnFace end;
			8: begin label9.color := clRED; label8.color := clBtnFace end;
			9: begin label10.color := clRED; label9.color := clBtnFace end;
		end;
		for nc := 0 to StrLen(buf)-1 do begin
                        ch := buf[nc];
			FileWrite(TTYHandle, ch, 1);
			t := time;
			repeat
			   Application.ProcessMessages;
			until (time-t) > OneSec/10;
		end;
		t := time;
		repeat
			Application.ProcessMessages;
		until (time-t) > OneSec/4;
            end;
	end;
	label10.color := clBtnFace;
        { Remove commands echo from RX buffer }
        EmptyBuffer;

        { Re-read and display channels }
	if ReadChannels then begin
            ParseChannels;
            DisplayChannels;
        end;
    end else begin
        Application.MessageBox(
        'Can''t write; no data available',
        'Error!',[smbOK]);
    end;
end;

{
	Exit program
}
procedure TForm1.Exit1Click(Sender: TObject);
begin
    if TTYHandle > 0 then FileClose(TTYHandle);
	Halt;
end;

{
	Things to do at startup
		- read COM port number from configuration file
		  and setup menus to reflect configuration
		- open selected COM port
		- read, parse and display channel assignment
}
procedure TForm1.FormCreate(Sender: TObject);

var ConfFile: FILE of integer;

begin
	AssignFile(ConfFile,'T7F.INI');
        try
            Reset(ConfFile);
            Read(ConfFile,TTYPort);
            Read(ConfFile,ChanSpc);
            CloseFile(ConfFile);
        except
	    TTYPort := 0;
	    SaveConfig;
	end;
	case TTYPort of
		0: begin
			Com11.Checked := TRUE;
			Com21.Checked := FALSE;
			Com31.Checked := FALSE;
			Com41.Checked := FALSE;
			TTYPort := 0;
                        TTYHandle := FileOpen('/dev/ttyS0', O_RDWR);
		end;
		1: begin
			Com11.Checked := FALSE;
			Com21.Checked := TRUE;
			Com31.Checked := FALSE;
			Com41.Checked := FALSE;
			TTYPort := 1;
                        TTYHandle := FileOpen('/dev/ttyS1', O_RDWR);
		end;
		2: begin
			Com11.Checked := FALSE;
			Com21.Checked := FALSE;
			Com31.Checked := TRUE;
			Com41.Checked := FALSE;
			TTYPort := 2;
                        TTYHandle := FileOpen('/dev/ttyS2', O_RDWR);
		end;
		3: begin
			Com11.Checked := FALSE;
			Com21.Checked := FALSE;
			Com31.Checked := FALSE;
			Com41.Checked := TRUE;
                        TTYPort := 3;
                        TTYHandle := FileOpen('/dev/ttyS3', O_RDWR);
		end;
	end;
	if ChanSpc = 25000 then begin
		N125kHz1.Checked := False;
		N25kHz1.Checked := True;
	end else begin
		N125kHz1.Checked := True;
		N25kHz1.Checked := False;
	end;
	if TTYHandle <= 0 then begin
		StrFmt(sz, 'Can''t open ttyS%d',[TTYPort]);
		Application.MessageBox(sz,'Error!',[smbOK],smsCritical);
	end;
	SetComParams;
        ClearDisplay;
	if ReadChannels then begin
        	ParseChannels;
	        DisplayChannels;
        end;
end;

{
	Things to do when entering Rx and TX frequency edit boxes
}
procedure TForm1.RxTXEnter(Sender: TObject);
begin
	(Sender as TMaskEdit).SelStart := 2;
	(Sender as TMaskEdit).SelLength := 1;
end;

{
	Things to do when exiting from Rx and Tx frequency edit boxes
}
procedure TForm1.RxTxExit(Sender: TObject);

begin
	CheckFreq(Sender as TMaskEdit);
end;

{
	Select 12.5 kHz channel spacing
}
procedure TForm1.N125kHz1Click(Sender: TObject);
begin
	ChanSpc := 12500;
	N125kHz1.Checked := True;
	N25kHz1.Checked := False;
	SaveConfig;
	if DataValid then DisplayChannels;
end;

{
	Select 25 kHz channel spacing
}
procedure TForm1.N25kHz1Click(Sender: TObject);
begin
	ChanSpc := 25000;
	N125kHz1.Checked := False;
	N25kHz1.Checked := True;
	SaveConfig;
	if DataValid then DisplayChannels;
end;

{
	Load channel file from disk
}
procedure TForm1.Load1Click(Sender: TObject);

var ChanFile: FILE of TChanConf;

begin
        DataValid := False;
	if OpenDialog1.Execute then begin
            try
		AssignFile(ChanFile,OpenDialog1.Filename);
		Reset(ChanFile);
		Read(ChanFile,ChanConf);
		CloseFile(ChanFile);
		ChanSpc := ChanConf.ChanSpc;
		RxCh := ChanConf.RxCh;
		TxCh := ChanConf.TxCh;
		if (ChanSpc = 25000) or (ChanSpc = 12500) then begin
                    if ChanSpc = 25000 then begin
			N125kHz1.Checked := False;
			N25kHz1.Checked := True;
                    end;
                    if ChanSpc = 12500 then begin
			N125kHz1.Checked := True;
			N25kHz1.Checked := False;
                    end;
                    DisplayChannels;
                    DataValid := True;
                end else begin
                    Application.MessageBox(
                        'Illegal channel file format','Error!',[smbOK],smsCritical);
                    DataValid := False;
                    FillChar(ChanConf, SizeOf(ChanConf),$FF);
                    ChanConf.ChanSpc := ChanSpc;
                    ClearDisplay;
                end;
            except
                on Exception do begin
                    Application.MessageBox(
                        'Can''t open file','Error!',[smbOK],smsCritical);
                    DataValid := False;
                    FillChar(ChanConf, SizeOf(ChanConf),$FF);
                    ChanConf.ChanSpc := ChanSpc;
                    ClearDisplay;
                end;
            end;
        end;
end;

{
	Save channel file to disk
}
procedure TForm1.Save1Click(Sender: TObject);

var ChanFile: FILE of TChanConf;

begin
	if DataValid and SaveDialog1.Execute then begin
		GetFreq;
		ChanConf.ChanSpc := ChanSpc;
		ChanConf.RxCh := RxCh;
		ChanConf.TxCh := TxCh;
                try
                    AssignFile(ChanFile,SaveDialog1.Filename);
                    ReWrite(ChanFile);
        	    Write(ChanFile, ChanConf);
		    CloseFile(ChanFile);
                except
                    on Exception do begin
        	        Application.MessageBox('Can''t write channel file',
	                'Error!',[smbOK]);
                    end;
		end;
	end else begin
            Application.MessageBox('Can''t write: no data available',
            'Error!',[smbOK], smsCritical);
        end;
end;

{
	Check if entered frequency is valid
	Must be an integer multiple of the channel spacing
}
procedure Tform1.CheckFreq(EdBox: TmaskEdit);

var kHz, Hz: longint;
	 ErrC: integer;

begin
	Val(Copy(EdBox.Text, 5, 3), kHz, ErrC);
	Val(Copy(EdBox.Text, 9, 1), Hz, ErrC);
	if ((kHz*1000+100*Hz) mod ChanSpc) <> 0 then begin
		Application.MessageBox(
		'Invalid frequency (not an integer multiple of channel spacing)',
		'Error!', [smbOK], smsCritical);
		EdBox.SetFocus;
		EdBox.SelStart := 2;
		EdBox.SelLength := 1;
                DataValid := False;
	end else begin
                DataValid := True;
        end;
end;

{
	Get and show 40 bytes channel hexdump
}
procedure TForm1.HexDump1Click(Sender: TObject);

var p1,p2: pchar;

begin
        if ReadChannels then begin
        	p1 := RxBuf;
	        p2 := RxBuf;
        	FrHexDump.Memo1.Lines.Clear;
        	while p1^ <> chr(0) do begin
                    if p1^ = chr(13) then begin
                        p1^ := chr(0);
			FrHexDump.Memo1.Lines.Add(StrPas(p2));
        		inc(p1);
			p2 := p1;
                    end else inc(p1);
                end;
            FrHexDump.Visible := TRUE;
            FrHexDump.BringToFront;
        end;
end;

{
	Show GNU General Public License
}
procedure TForm1.GNULicense1Click(Sender: TObject);
begin
	GNULicense.Visible := TRUE;
end;

{
	Show T7F connectors pin assignment
}
procedure TForm1.Connectorspinout1Click(Sender: TObject);
begin
	FrPinout.Visible := TRUE;
end;

{
	Do not allow window resizing
}
procedure TForm1.FormResize(Sender: TObject);
begin
	Form1.Height := 409;
	Form1.Width :=  329;
end;

{
	Init default values
} procedure TForm1.New1Click(Sender: TObject);
begin
    FillChar(ChanConf, SizeOf(ChanConf),$FF);
    ChanConf.ChanSpc := ChanSpc;
    (Form1.ActiveControl as TMaskEdit).Text := '430.000.0';
    Form1.ActiveControl := Rx0;
    ClearDisplay;
    Rx0.SelStart := 2;
    Rx0.SelLength := 1;
    DataValid := False;
end;


procedure TForm1.About1Click(Sender: TObject);
begin
    AboutBox.Visible := True;
end;


Initialization
	begin
                ChanSpc := 12500;
                DataValid := False;
	end;
end.
